MODULE ldftra_smag
   !!======================================================================
   !!                     ***  MODULE  ldftrasmag  ***
   !! Ocean physics:  variable eddy induced velocity coefficients
   !!======================================================================
#if   defined key_traldf_smag   &&   defined key_traldf_c3d
   !!----------------------------------------------------------------------
   !!   'key_traldf_smag'      and           smagorinsky  diffusivity
   !!   'key_traldf_c3d'                    3D tracer lateral  mixing coef.
   !!----------------------------------------------------------------------
   !!   ldf_eiv      : compute the eddy induced velocity coefficients
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE sbc_oce         ! surface boundary condition: ocean
   USE sbcrnf          ! river runoffs
   USE ldftra_oce      ! ocean tracer   lateral physics
   USE phycst          ! physical constants
   USE ldfslp          ! iso-neutral slopes
   USE in_out_manager  ! I/O manager
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE prtctl          ! Print control
   USE iom

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC ldf_tra_smag               ! routine called by step.F90
   !!----------------------------------------------------------------------
   !!  OPA 9.0 , LOCEAN-IPSL (2005) 
   !! $Id: ldf_tra_smag.F90 1482 2010-06-13 15:28:06Z  $
   !! This software is governed by the CeCILL licence see modipsl/doc/NEMO_CeCILL.txt 
   !!----------------------------------------------------------------------
   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------

CONTAINS

 



  !!----------------------------------------------------------------------
  !!                        ***  ldf_tra_smag.F90  ***
  !!----------------------------------------------------------------------


   SUBROUTINE ldf_tra_smag( kt )
      !!----------------------------------------------------------------------
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE ldf_tra_smag  ***
      !!                   
      !! ** Purpose :   initializations of the horizontal ocean physics
      !!
      !! ** Method  :   3D eddy viscosity coef. 
      !!    M.Griffies, R.Hallberg AMS, 2000
      !! for laplacian:
      !!   Asmag=(C/pi)^2*dx*dy sqrt(D^2), C=1 for tracers, C=3-4 for viscosity
      !! for bilaplacian:
      !!   Bsmag=Asmag*dx*dy/8
      !!   D^2=(du/dx-dv/dy)^2+(dv/dx+du/dy)^2 for Cartesian coordinates
      !!  in general case du/dx ==> e2 d(u/e2)/dx;  du/dy ==> e1 d(u/e1)/dy; 
      !!                  dv/dx ==> e2 d(v/e2)/dx;  dv/dy ==> e1 d(v/e1)/dy
      !!  
      !!       laplacian operator   : ahm1, ahm2 defined at T- and F-points
      !!                              ahm3, ahm4 never used
      !!       bilaplacian operator : ahm1, ahm2 never used
      !!                           :  ahm3, ahm4 defined at U- and V-points
      !!       ??? explanation of the default is missing
      !!  last modified : Maria Luneva, September 2011
      !!----------------------------------------------------------------------
      !!
      !!----------------------------------------------------------------------
      !! * Modules used
      USE ioipsl
      INTEGER, INTENT( in ) ::   kt     ! ocean time-step inedx
      !! * Arguments
      INTEGER               :: ji,jj,jk
      REAL (wp), DIMENSION (:,:), ALLOCATABLE::    ux,uy,vx,vy     ! local velocity derivatives
      REAL (wp), DIMENSION (:,:), ALLOCATABLE::    ue1,ue2,ve1,ve2 ! local variables
      REAL (wp)                     ::    deltaxy, deltau, deltav  ! local step 

      !!----------------------------------------------------------------------
      IF(  kt == nit000 ) THEN
      IF( lk_traldf_eiv ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' ldf_tra_smag : 3D eddy smagorinsky diffusivity and eddy induced velocity coefficients'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~   --  '
         IF(lwp) WRITE(numout,*) '               Coefficients are computed'
         IF(lwp) WRITE(numout,*)
      ELSE
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' ldf_tra_smag : 3D eddy smagorinsky diffusivity coefficient'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~   -- '
         IF(lwp) WRITE(numout,*) '               Coefficients are computed'
         IF(lwp) WRITE(numout,*)
      ENDIF
      ENDIF

      ALLOCATE(ux(jpi,jpj)) ; ux(:,:)=0._wp
      ALLOCATE(uy(jpi,jpj)) ; uy(:,:)=0._wp
      ALLOCATE(vx(jpi,jpj)) ; vx(:,:)=0._wp
      ALLOCATE(vy(jpi,jpj)) ; vy(:,:)=0._wp
      ALLOCATE(ue1(jpi,jpj)); ALLOCATE(ue2(jpi,jpj)) 
      ALLOCATE(ve1(jpi,jpj)); ALLOCATE(ve2(jpi,jpj)) 

      ! biharmonic operator   (T-point)
      ! -------------------
     
      ahtt(:,:,:) = aht0              ! set ahtt at T-point (here no space variation)
        IF( ln_traldf_bilap ) THEN
         ! define ahm1 and ahm2 at the right grid point position
         ! (USER: modify ahm1 and ahm2 following your desiderata)
         DO jk=1,jpk
           ue2(:,:)=un(:,:,jk)/e2u(:,:)
           ve1(:,:)=vn(:,:,jk)/e1v(:,:)
           ue1(:,:)=un(:,:,jk)/e1u(:,:)
           ve2(:,:)=vn(:,:,jk)/e2v(:,:)


           DO jj=2,jpj
            DO ji=2,jpi
            ux(ji,jj)=(ue2(ji,jj)-ue2(ji-1,jj))/e1t(ji,jj)*e2t(ji,jj)*tmask(ji,jj,jk)
            vy(ji,jj)=(ve1(ji,jj)-ve1(ji,jj-1))/e2t(ji,jj)*e1t(ji,jj)*tmask(ji,jj,jk)
            ENDDO
           ENDDO

           DO jj=1,jpjm1
            DO ji=1,jpim1
            uy(ji,jj)=(ue1(ji,jj+1)-ue1(ji,jj))/e2f(ji,jj)*e1f(ji,jj)*fmask(ji,jj,jk)
            vx(ji,jj)=(ve2(ji+1,jj)-ve2(ji,jj))/e1f(ji,jj)*e2f(ji,jj)*fmask(ji,jj,jk)
            ENDDO
           ENDDO


          DO jj=2,jpjm1
           DO ji=2,jpim1
            deltaxy=2./(e1t(ji,jj)**(-2)+e2t(ji,jj)**(-2))
             ahtt(ji,jj,jk)=(chsmag/3.14)**2/8*deltaxy**2*                      &
                            sqrt( (ux(ji,jj)-vy(ji,jj))**2+                              &
                     0.0625*(uy(ji,jj)+uy(ji,jj-1)+uy(ji-1,jj)+uy(ji-1,jj-1)+            &
                        vx(ji,jj)+vx(ji,jj-1)+vx(ji-1,jj)+vx(ji-1,jj-1) )**2)
   
              !!! stabulity criteria: abs(aht)<delta**4/(4*8*dt)   dt=2*rdt 
             ahtt(ji,jj,jk)=MIN( ahtt(ji,jj,jk) , aht0 )      
             ahtt(ji,jj,jk)=MAX( ahtt(ji,jj,jk) ,- deltaxy**4/(8*16*rdt) ) 
            ENDDO
           ENDDO
         ENDDO
            ahtt(:,:,jpk) = ahtt(:,:,jpkm1)

      CALL lbc_lnk( ahtt, 'T', 1. )   ! Lateral boundary conditions on ( ahtt )
IF(  kt == nit000 ) THEN

      IF(lwp ) THEN                    ! Control print
         WRITE(numout,*)
         WRITE(numout,*) 'inildf: ahtt at k = 1'
         CALL prihre( ahtt(:,:,1), jpi, jpj, 1, jpi, 1,   &
            &                                1, jpj, 1, 1.e-1, numout )

      ENDIF

      ENDIF
ENDIF



      ! harmonic operator   (U-, V-, W-points)
      ! ----------------- 

      ahtu(:,:,:) = aht0              ! set ahtu = ahtv at u- and v-points,
      ahtv(:,:,:) = aht0              ! and ahtw at w-point
      ahtw(:,:,:) = aht0              ! (here example: no space variation)

      IF( ln_traldf_lap ) THEN
         DO jk=1,jpk
           ue2(:,:)=un(:,:,jk)/e2u(:,:)
           ve1(:,:)=vn(:,:,jk)/e1v(:,:)
           ue1(:,:)=un(:,:,jk)/e1u(:,:)
           ve2(:,:)=vn(:,:,jk)/e2v(:,:)


           DO jj=2,jpj
            DO ji=2,jpi
            ux(ji,jj)=(ue2(ji,jj)-ue2(ji-1,jj))/e1t(ji,jj)*e2t(ji,jj)*tmask(ji,jj,jk)
            vy(ji,jj)=(ve1(ji,jj)-ve1(ji,jj-1))/e2t(ji,jj)*e1t(ji,jj)*tmask(ji,jj,jk)
            ENDDO
           ENDDO

           DO jj=1,jpjm1
            DO ji=1,jpim1
            uy(ji,jj)=(ue1(ji,jj+1)-ue1(ji,jj))/e2f(ji,jj)*e1f(ji,jj)*fmask(ji,jj,jk)
            vx(ji,jj)=(ve2(ji+1,jj)-ve2(ji,jj))/e1f(ji,jj)*e2f(ji,jj)*fmask(ji,jj,jk)
            ENDDO
           ENDDO


          DO jj=2,jpjm1
           DO ji=2,jpim1
           deltau=2./(e1u(ji,jj)**(-2)+e2u(ji,jj)**(-2))
           deltav=2./(e1v(ji,jj)**(-2)+e2v(ji,jj)**(-2))

           ahtu(ji,jj,jk)=MAX(aht0 , (chsmag/3.14)**2*deltau*            &
                         sqrt(0.25*(ux(ji,jj)+ux(ji+1,jj)-vy(ji,jj)-vy(ji+1,jj))**2+    &
                              0.25*(uy(ji,jj)+uy(ji,jj-1)+vx(ji,jj)+vx(ji,jj-1))**2))

           ahtv(ji,jj,jk)=MAX(aht0 , (chsmag/3.14)**2*deltav*            &
                         sqrt(0.25*(ux(ji,jj)+ux(ji,jj+1)-vy(ji,jj)-vy(ji,jj+1))**2+    &
                              0.25*(uy(ji,jj)+uy(ji-1,jj)+vx(ji-1,jj)+vx(ji,jj))**2))


         !!! stabulity criteria: aht<delta**2/(4*dt)   dt=2*rdt

             ahtu(ji,jj,jk)=MIN(ahtu(ji,jj,jk),deltau**2/(16*rdt) )
             ahtv(ji,jj,jk)=MIN(ahtv(ji,jj,jk),deltav**2/(16*rdt) )

            ENDDO
           ENDDO
         ENDDO
        ENDIF
            ahtu(:,:,jpk) = ahtu(:,:,jpkm1)
            ahtv(:,:,jpk) = ahtv(:,:,jpkm1)
        CALL lbc_lnk( ahtu, 'U', 1. )   ! Lateral boundary conditions
        CALL lbc_lnk( ahtv, 'V', 1. )


          DEALLOCATE( ux ) ;DEALLOCATE( uy );DEALLOCATE( vx );DEALLOCATE( vy )     
          DEALLOCATE( ue1 ) ;DEALLOCATE( ue2 );DEALLOCATE( ve1 );DEALLOCATE( ve2 )

IF(  kt == nit000 ) THEN

      IF(lwp ) THEN                    ! Control print
         WRITE(numout,*)
         WRITE(numout,*) 'inildf: ahtu at k = 1'
         CALL prihre( ahtu(:,:,1), jpi, jpj, 1, jpi, 1,   &
            &                                1, jpj, 1, 1.e-1, numout )
         WRITE(numout,*)
         WRITE(numout,*) 'inildf: ahtv at k = 1'
         CALL prihre( ahtv(:,:,1), jpi, jpj, 1, jpi, 1,   &
            &                                1, jpj, 1, 1.e-1, numout )
         WRITE(numout,*)
         WRITE(numout,*) 'inildf: ahtw at k = 1'
         CALL prihre( ahtw(:,:,1), jpi, jpj, 1, jpi, 1,   &
            &                                1, jpj, 1, 1.e-1, numout )
      ENDIF
ENDIF

END SUBROUTINE ldf_tra_smag
#else
   !!----------------------------------------------------------------------
   !!   Default option                                         Dummy module
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE ldf_tra_smag( kt )       ! Empty routine
      WRITE(*,*) 'ldf_dyn_smag: You should not have seen this print! error? check keys ldf:c3d+smag', kt
   END SUBROUTINE ldf_tra_smag
#endif

END MODULE ldftra_smag
